/*
  eagle - controls and limits the programs in execution

  Copyright (C) 2000 <Tullio Dovera tdovera@libero.it>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Send any bug reports or comments to tdovera@libero.it
*/

#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include <ctype.h>
#include <signal.h>
#include <syslog.h>
#include <string.h>

#include "arg.h"
#include "rules.h"
#include "sysLog.h"

#define MAX_PROCS 512

struct proc {
  int  pid;
  char name[256];
  int  uid;
  int  gid;
};

struct proc procsStart[MAX_PROCS];
struct proc procs[MAX_PROCS];

int isnumber(char *s) {
  int status=0;
  
  while((*s) && (status=isdigit(*s)))
    s++;
  return status;
}

void listProcesses(struct proc procs[MAX_PROCS])
{
  DIR *dir;
  struct dirent *entry;
  char file[256];
  int i=0;
  FILE *fd;
  char line[256];
  char desc[64],value[64];

  dir=opendir("/proc");
  while ((entry=readdir(dir))!=NULL) {
    if (isnumber(entry->d_name)!=0) {
      /* insert PID */
      procs[i].pid=atoi(entry->d_name);
      /* insert uid, progname */
      sprintf(file,"/proc/%d/status",procs[i].pid);
      if ((fd=fopen(file,"r")) != NULL) {
	while (fgets(line,sizeof(line),fd)!=NULL) {
	  sscanf(line,"%s %s",desc,value);
	  if (strcmp(desc,"Name:")==0)
	    strcpy(procs[i].name,value);
	  if (strcmp(desc,"Uid:")==0)
	    procs[i].uid=atoi(value);
	  if (strcmp(desc,"Gid:")==0)
	    procs[i].gid=atoi(value);
	}
	fclose(fd);
	i++;
      }
    }
  }
  closedir(dir);
  procs[i].pid=0;
}

/*

  return:
  1 if exist
  0 otherwise
*/
int existPid(struct proc procs[], int pid) {

  int j=0;
  
  while(procs[j].pid!=0) {
    if (procs[j].pid==pid) {
      return 1;
    }
    j++;
  }
  return 0;
}

int main(int argc,char **argv,char **envp)
{
  int i;
  int myPid;
  char command[64];
  FILE *fd;
  int status;
  char buffer[256];

  myPid=getpid();

  controlArgument(argc, argv);

  loadRules(configFile());

  if (existingProcesses())
    listProcesses(procsStart);
  else
    procsStart[0].pid=-1;

  while(1) {
    sleep(updateSeconds());
    listProcesses(procs);
    i=0;
    while(procs[i].pid!=0) {
      if (procs[i].pid==myPid) {
	if (verboseMode())
	  printf("Eagle daemon      pid=%5d uid=%5d gid=%5d name=%s\n",
		 procs[i].pid,procs[i].uid,procs[i].gid,procs[i].name);
	i++;
	continue;
      }
      if (existPid(procsStart,procs[i].pid)) {
	if (verboseMode())
	  printf("already existing  pid=%5d uid=%5d gid=%5d name=%s\n",
		 procs[i].pid,procs[i].uid,procs[i].gid,procs[i].name);
	i++;
	continue;
      }
      if ((status=ruleOK(procs[i].uid,procs[i].gid,procs[i].name))) {
	if (verboseMode())
	  switch(status) {
	  case 1:
	    printf("granted user    pid=%5d uid=%5d gid=%5d name=%s\n",
		   procs[i].pid,procs[i].uid,procs[i].gid,procs[i].name);
	    break;
	  case 2:
	    printf("granted group   pid=%5d uid=%5d gid=%5d name=%s\n",
		   procs[i].pid,procs[i].uid,procs[i].gid,procs[i].name);
	    break;
	  }
	i++;
	continue;
      }
      sprintf(buffer,"kill          pid=%5d uid=%5d gid=%5d name=%s\n",
	       procs[i].pid,procs[i].uid,procs[i].gid,procs[i].name);
      sysLog(progName(),buffer);
      if (verboseMode())
	printf(buffer);
      kill(procs[i].pid,9);
      if (strcmp(mailTo(),"")!=0) {
	sprintf(command,"mail -s %s %s",progName(),mailTo());
	if ((fd=popen(command,"w"))==NULL)
	  perror(command);
	else {
	  fprintf(fd,buffer);
	  pclose(fd);
	}
      }
      i++;
    }
  }
  return 0;
}
